function [NewBdyEdges UnusedEdges]= BdyEdgesUpdate(EdgeListT, BdyEdges, e, UnusedEdges)
% [NewBdyEdges UnusedEdges]= BdyEdgesUpdate(EdgeListT, BdyEdges, e, UnusedEdges)

%Since all boundary edges to be added or removed with be adjacent to the
%non-tree endpoint of the edge e, we first determine this non-tree
%endpoint, which we denote as nonTreeEndpt
if ismember(e(1),EdgeListT)
    nonTreeEndpt = e(2);
else
    nonTreeEndpt = e(1);
end

if nargin < 6 %if UnusedEdges were not initialized, the program needs to create them
    
    %We first form the UnusedEdges to be those edges of G that do not appear
    %as tree edges, and do not appear as boundary edges.  Note that once a 
    %boundary edge has been added, it can be deleted, but will never be added a 
    %second time.
    UnusedEdges = setdiff(EdgeListG, EdgeListT, 'rows');
    UnusedEdges = setdiff(UnusedEdges, BdyEdges, 'rows');
    %We also need to delete any edge from UnusedEdges both of whose endpoints
    %are in the tree (These edges have not yet been accounted for, and will
    %subsequently never be boundary edges.)
    [r c] = size(UnusedEdges);
    CycleEdge = []; %this vector will collect all Unused edges both of whose
    %endpoints are tree endpoints, and thus would form a cyle in in T
    for i = 1:r
        if ismember(UnusedEdges(i,1),EdgeListT) & ismember(UnusedEdges(i,1),EdgeListT)
            CycleEdge = [CycleEdge i];
        end
    end
    UnusedEdges(CycleEdge,:)=[];
end
    

%2.  We need to delete any edge of BdyEdges that has nonTreeEndpt
%as one of its endpoints.
[a b] = find(BdyEdges == nonTreeEndpt);
BdyEdges(a,:) = [];
%3.  Next, we find the unused edges that are incident to nonTreeEndpt.  The
%vector a below gives their row indices in UnusedEdges, and the vector b
%gives the columnn index(1 or 2) in which nonTreeEndpt occurs.
[a b] = find(nonTreeEndpt==UnusedEdges);
%We add these edges to BdyEdges
BdyEdges = [BdyEdges; UnusedEdges(a,:)];
%and delete them from UnusedEdges
UnusedEdges(a,:)=[];
NewBdyEdges = BdyEdges;



